MAP = {
    "user_id": (
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/nameidentifier"
    ),
    "email": (
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress"
    ),
    "name": "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name",
    "given_name": (
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/givenname"
    ),
    "family_name": (
        "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/surname"
    ),
    "upn": "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/upn",
    "groups": "http://schemas.xmlsoap.org/claims/Group",
}



from __future__ import annotations

from datetime import datetime, timedelta

import ckan.plugins as plugins
import ckan.plugins.toolkit as tk
from flask import session

from ckanext.saml.cli import get_commnads
from ckanext.saml.helpers import get_helpers
from ckanext.saml.logic.action import get_actions
from ckanext.saml.views import saml
import ckanext.saml.config as config


class SamlPlugin(plugins.SingletonPlugin):
    plugins.implements(plugins.IActions)
    plugins.implements(plugins.IConfigurer)
    plugins.implements(plugins.IBlueprint)
    plugins.implements(plugins.IClick)
    plugins.implements(plugins.ITemplateHelpers)

    # IActions
    def get_actions(self):
        return get_actions()

    # IConfigurer

    def update_config(self, config_):
        tk.add_template_directory(config_, "templates")

    # ITemplateHelpers

    def get_helpers(self):
        return get_helpers()

    if not tk.check_ckan_version("2.10"):
        # specifically for ckan 2.9 and lower versions
        plugins.implements(plugins.IAuthenticator, inherit=True)
        # IAuthenticator

        def identify(self):
            if "samlCKANuser" not in session:
                return

            now = datetime.utcnow()
            last_login = session.get("samlLastLogin", now)
            diff = now - last_login

            ttl = tk.asint(tk.config.get(
                config.CONFIG_TTL, config.DEFAULT_TTL))
            if diff < timedelta(seconds=ttl):
                tk.g.user = session["samlCKANuser"]

        def logout(self):
            if "samlNameId" in session:
                for key in saml.saml_details:
                    del session[key]

    # IBlueprint
    def get_blueprint(self):
        return [saml.get_bp()]

    # IClick
    def get_commands(self):
        return get_commnads()



from __future__ import annotations

import logging
import uuid
from datetime import datetime, timedelta

import ckan.lib.helpers as h
from ckan.logic import functools
import ckan.model as model
import ckan.plugins as plugins
import ckan.plugins.toolkit as tk
from ckan.logic.action.create import _get_random_username_from_email
from flask import Blueprint, make_response, session

from sqlalchemy import func as sql_func

from ckanext.saml.interfaces import ICKANSAML
from ckanext.saml.model.user import User
from ckanext.saml import utils, config


log = logging.getLogger(__name__)
use_nameid_as_email = tk.asbool(
    tk.config.get("ckan.saml_use_nameid_as_email", False)
)

saml_details = [
    "samlUserdata",
    "samlNameIdFormat",
    "samlNameId",
    "samlCKANuser",
]

saml = Blueprint("saml", __name__)


@functools.lru_cache(1)
def get_bp():
    saml.add_url_rule(config.slo_path(), view_func=post_logout)

    saml.add_url_rule(
        config.sso_path(), view_func=post_login, methods=["POST"]
    )

    return saml


def post_logout():
    if "SAMLResponse" in tk.request.args:
        log.debug(
            "SAML2 Logout response: %s",
            utils.decode_saml_response(tk.request.args["SAMLResponse"])
        )
    return tk.h.redirect_to("user.logout")


def post_login():
    req = utils.prepare_from_flask_request()
    auth = utils.make_auth(req)

    request_id = None
    auth.process_response(request_id=request_id)
    errors = auth.get_errors()

    if errors:
        log.error("{}".format(errors))
        error_tpl = config.error_template()
        if error_tpl:
            return tk.render(error_tpl, {"errors": errors})

        h.flash_error("Login failed.")
        return h.redirect_to(h.url_for("saml.saml_login"))

    log.debug("User succesfully logged in the IdP. Extracting NAMEID.")
    nameid = auth.get_nameid()

    if not nameid:
        log.error(
            "Something went wrong, no NAMEID was found, "
            "redirecting back to to login page."
        )
        return h.redirect_to(h.url_for("user.login"))

    mapped_data = {}
    attr_mapper = tk.h.saml_attr_mapper()

    if not attr_mapper:
        log.error(
            'User mapping is empty, please set "ckan.saml_custom_attr_map"'
            " param in config."
        )
        return h.redirect_to(h.url_for("user.login"))

    for key, value in attr_mapper.items():
        field = auth.get_attribute(value)
        if field:
            mapped_data[key] = field
    log.debug("NAMEID: {0}".format(nameid))

    for item in plugins.PluginImplementations(ICKANSAML):
        item.after_mapping(mapped_data, auth)
    log.debug("Client data: %s", attr_mapper)
    log.debug("Mapped data: %s", mapped_data)
    log.debug(
        "If you are experiencing login issues, make sure that email is present"
        " in the mapped data"
    )
    saml_user = (
        model.Session.query(User).filter(User.name_id == nameid).first()
    )

    if not saml_user:
        log.debug(
            "No User with NAMEID '{0}' was found. Creating one.".format(nameid)
        )

        try:
            email = (
                nameid
                if config.use_nameid_as_email()
                else mapped_data["email"][0]
            )

            log.debug(
                'Check if User with "{0}" email already exists.'.format(email)
            )
            user_exist = (
                model.Session.query(model.User)
                .filter(
                    sql_func.lower(model.User.email) == sql_func.lower(email)
                )
                .first()
            )

            if user_exist:
                log.debug(
                    'Found User "{0}" that has same email.'.format(
                        user_exist.name
                    )
                )
                new_user = user_exist.as_dict()
                log_message = (
                    "User is being detected with such NameID, adding to Saml2"
                    " table..."
                )
            else:
                user_dict = {
                    "name": _get_random_username_from_email(email),
                    "email": email,
                    "id": str(uuid.uuid4()),
                    "password": str(uuid.uuid4()),
                    "fullname": mapped_data["fullname"][0]
                    if mapped_data.get("fullname")
                    else "",
                }

                log.debug(
                    "Trying to create User with name '{0}'".format(
                        user_dict["name"]
                    )
                )

                new_user = tk.get_action("user_create")(
                    {"ignore_auth": True}, user_dict
                )
                log_message = "User succesfully created. Authorizing..."

            # Make sure that User ID is not already in saml2_user table
            saml_user = (
                model.Session.query(User)
                .filter(User.id == new_user["id"])
                .first()
            )

            if saml_user:
                log.debug(
                    "Found existing row with such User ID, updating NAMEID..."
                )
                saml_user.name_id = nameid
            else:
                saml_user = User(
                    id=new_user["id"],
                    name_id=nameid,
                    attributes=mapped_data,
                )
                model.Session.add(saml_user)
            model.Session.commit()
            log.debug(log_message)
            user = model.User.get(new_user["name"])
        except Exception as e:
            log.exception("Cannot create SAML2 user")
            return h.redirect_to(h.url_for("user.login"))
    else:
        user = model.User.get(saml_user.id)

    user_dict = user.as_dict()
    saml_user.attributes = mapped_data

    # Compare User data if update is needed.
    check_fields = ["fullname"]
    update_dict = {}

    for field in check_fields:
        if mapped_data.get(field):
            updated = (
                True if mapped_data[field][0] != user_dict[field] else False
            )
            if updated:
                update_dict[field] = mapped_data[field][0]

    if user_dict["state"] == "deleted":
        if config.reactivate_deleted_account():
            update_dict["state"] = "active"
            log.debug(
                "Restore deleted user %s",
                user_dict["name"]
            )

        else:
            log.warning(
                "Blocked login attempt for deleted user %s",
                user_dict["name"]
            )

            h.flash_error(
                tk._("Your account was deleted. Please, contact the administrator if you want to restore it")
            )
            return tk.abort(403)

    if update_dict:
        for item in update_dict:
            user_dict[item] = update_dict[item]

        tk.get_action("user_update")({"ignore_auth": True}, user_dict)
    model.Session.commit()

    # Roles and Organizations
    for item in plugins.PluginImplementations(ICKANSAML):
        item.roles_and_organizations(mapped_data, auth, user)


    if tk.check_ckan_version("2.10"):
        duration_time = timedelta(milliseconds=int(tk.config.get(
                config.CONFIG_TTL, config.DEFAULT_TTL)))

        tk.login_user(user, duration=duration_time)
    else:
        session["samlUserdata"] = auth.get_attributes()
        session["samlNameIdFormat"] = auth.get_nameid_format()
        session["samlNameId"] = nameid
        session["samlCKANuser"] = user.name
        session["samlLastLogin"] = datetime.utcnow()

        tk.g.user = user.name

    if "RelayState" in req["post_data"] and req["post_data"]["RelayState"]:
        log.info('Redirecting to "{0}"'.format(req["post_data"]["RelayState"]))
        return h.redirect_to(req["post_data"]["RelayState"])

    return tk.redirect_to(_destination())

@saml.route("/saml/metadata")
def metadata():
    try:
        context = dict(model=model, user=tk.g.user, auth_user_obj=tk.g.userobj)
        tk.check_access("sysadmin", context)
    except tk.NotAuthorized:
        tk.abort(403, tk._("Need to be system administrator to administer"))

    req = utils.prepare_from_flask_request()
    auth = utils.make_auth(req)

    settings = auth.get_settings()
    metadata = settings.get_sp_metadata()
    errors = settings.validate_metadata(metadata)

    if len(errors) == 0:
        resp = make_response(metadata, 200)
        resp.headers["Content-Type"] = "text/xml"
    else:
        resp = make_response(", ".join(errors), 500)
    return resp


@saml.route("/saml/login")
def saml_login():
    req = utils.prepare_from_flask_request()
    try:
        auth = utils.make_auth(req)

        if tk.asbool(tk.request.args.get("sso")) or config.unconditional_login():
            saml_relaystate = tk.config.get("ckan.saml_relaystate", None)
            redirect = (
                saml_relaystate
                if saml_relaystate
                else _destination()
            )
            if tk.request.args.get("redirect"):
                redirect = tk.request.args.get("redirect")

            if tk.g.user:
                return h.redirect_to(redirect)

            log.info("Redirect to SAML IdP.")
            return h.redirect_to(auth.login(return_to=redirect))
        else:
            log.warning(
                "No arguments been provided in this URL. If you want to make"
                " auth request to SAML IdP point, please provide '?sso=true'"
                " at the end of the URL."
            )
    except Exception as e:
        h.flash_error(
            "SAML: An issue appeared while validating settings file."
        )
        log.error("{}".format(e))

    return h.redirect_to(h.url_for("user.login"))


def _destination() -> str:
    dynamic = tk.request.args.get('came_from', '')
    static = tk.config.get('ckan.route_after_login', 'dashboard.index')
    return dynamic or static



# -*- coding: utf-8 -*-
from typing import Optional

from ckan.plugins import Interface
from onelogin.saml2.auth import OneLogin_Saml2_Auth


class ICKANSAML(Interface):
    """Implement custom SAML response modification."""

    def after_mapping(self, mapped_data, auth):
        """Return dictonary mapped fields.

        :returns: dictonary
        :rtype: dict

        """

        return mapped_data

    def roles_and_organizations(self, mapped_data, auth, user):
        """Map Roles and assign User to Organizations"""
        pass

    def saml_auth_class(self) -> Optional[OneLogin_Saml2_Auth]:
        """Custom SamlAuthenticator(subclass of OneLogin_Saml2_Auth)"""
        pass



"""empty message

Revision ID: 92745f8a6168
Revises:
Create Date: 2022-04-21 16:17:57.972546

"""
import sqlalchemy as sa
from alembic import op
from sqlalchemy.engine.reflection import Inspector

# revision identifiers, used by Alembic.
revision = "92745f8a6168"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    conn = op.get_bind()
    inspector = Inspector.from_engine(conn)
    tables = inspector.get_table_names()
    if "saml2_user" in tables:
        return

    op.create_table(
        "saml2_user",
        sa.Column("id", sa.UnicodeText, primary_key=True),
        sa.Column("name_id", sa.UnicodeText, unique=True, nullable=False),
        sa.Column("allow_update", sa.Boolean),
    )


def downgrade():
    op.drop_table(
        "saml2_user",
    )



from __future__ import annotations

import json
from typing import Any

import ckan.plugins.toolkit as tk
from ckan.lib.redis import connect_to_redis
from onelogin.saml2.idp_metadata_parser import (
    OneLogin_Saml2_IdPMetadataParser as Parser,
)

CONFIG_URL = "ckanext.saml.metadata.url"


def get_actions():
    return {
        "saml_idp_refresh": idp_refresh,
        "saml_idp_show": idp_show,
    }


def _idp_key():
    """Cache key for IdP details."""
    site_id = tk.config["ckan.site_id"]
    return "ckan:{}:saml:idp".format(site_id)


def _read_remote_metadata(path_or_url: str):
    if path_or_url.startswith("file://"):
        with open(path_or_url[len("file://"):]) as src:
            return Parser.parse(src.read())

    return Parser.parse_remote(path_or_url)


def idp_refresh(context: dict[str, Any], data_dict: dict[str, Any]):
    """Refresh IdP details using remote metadata."""
    tk.check_access("sysadmin", context, data_dict)

    url = data_dict.get("url", tk.config.get(CONFIG_URL))

    if not url:
        raise tk.ObjectNotFound(
            "Metadata URL is not configured: {}".format(CONFIG_URL)
        )
    meta = _read_remote_metadata(url)

    cache = connect_to_redis()
    cache.set(_idp_key(), json.dumps(meta["idp"]))
    return meta["idp"]


def idp_show(context: dict[str, Any], data_dict: dict[str, Any]):
    """Show IdP details pulled from the remote metadata."""
    tk.check_access("sysadmin", context, data_dict)
    cache = connect_to_redis()

    if value := cache.get(_idp_key()):
        return json.loads(value)

    return tk.get_action("saml_idp_refresh")(context, data_dict)



from __future__ import annotations
from typing import Optional

import ckan.plugins.toolkit as tk

CONFIG_ERROR_TPL = "ckanext.saml.error_template"

CONFIG_SSO_PATH = "ckanext.saml.sso_path"
DEFAULT_SSO_PATH = "/sso/post"

CONFIG_SLO_PATH = "ckanext.saml.slo_path"
DEFAULT_SLO_PATH = "/slo/post"

CONFIG_DYNAMIC = "ckanext.saml.settings.dynamic"
DEFAULT_DYNAMIC = False

CONFIG_USE_REMOTE_IDP = "ckanext.saml.metadata.remote_idp"
DEFAULT_USE_REMOTE_IDP = False

CONFIG_STATIC_HOST = "ckanext.saml.static_host"
DEFAULT_STATIC_HOST = None

CONFIG_USE_FORWARDED_HOST = "ckanext.saml.use_forwarded_host"
DEFAULT_USE_FORWARDED_HOST = False

CONFIG_UNCONDITIONAL_LOGIN = "ckanext.saml.unconditional_login"
DEFAULT_UNCONDITIONAL_LOGIN = False

CONFIG_LOGIN_TEXT = "ckanext.saml.login_button_text"
LEGACY_CONFIG_LOGIN_TEXT = "ckan.saml_login_button_text"
DEFAULT_LOGIN_TEXT = "SAML Login"

CONFIG_REACTIVATE = "ckanext.saml.reactivate_deleted_account"
DEFAULT_REACTIVATE = False

CONFIG_FOLDER_PATH = "ckanext.saml.metadata.base_path"
LEGACY_CONFIG_FOLDER_PATH = "ckan.saml_custom_base_path"
DEFAULT_FOLDER_PATH = "/etc/ckan/default/saml"

CONFIG_HTTPS = "ckan.saml_use_https"
DEFAULT_HTTPS = "off"

CONFIG_USE_NAMEID_AS_EMAIL = "ckan.saml_use_nameid_as_email"
DEFAULT_USE_NAMEID_AS_EMAIL = False

CONFIG_TTL = "ckanext.saml.session.ttl"
DEFAULT_TTL = 30 * 24 * 3600

def reactivate_deleted_account() -> bool:
    return tk.asbool(tk.config.get(CONFIG_REACTIVATE, DEFAULT_REACTIVATE))


def sso_path() -> str:
    return tk.config.get(CONFIG_SSO_PATH, DEFAULT_SSO_PATH)


def slo_path() -> str:
    return tk.config.get(CONFIG_SLO_PATH, DEFAULT_SLO_PATH)


def error_template() -> Optional[str]:
    return tk.config.get(CONFIG_ERROR_TPL)


def login_button_text() -> str:
    legacy = tk.config.get(LEGACY_CONFIG_LOGIN_TEXT)
    if legacy:
        return legacy

    return tk.config.get(CONFIG_LOGIN_TEXT, DEFAULT_LOGIN_TEXT)


def folder_path() -> str:
    legacy = tk.config.get(LEGACY_CONFIG_FOLDER_PATH)
    if legacy:
        return legacy

    return tk.config.get(CONFIG_FOLDER_PATH, DEFAULT_FOLDER_PATH)


def use_remote_idp() -> bool:
    return tk.asbool(
        tk.config.get(CONFIG_USE_REMOTE_IDP, DEFAULT_USE_REMOTE_IDP)
    )


def use_dynamic_config() -> bool:
    return tk.asbool(tk.config.get(CONFIG_DYNAMIC, DEFAULT_DYNAMIC))


def unconditional_login() -> bool:
    return tk.asbool(
        tk.config.get(CONFIG_UNCONDITIONAL_LOGIN, DEFAULT_UNCONDITIONAL_LOGIN)
    )


def use_forwarded_host() -> bool:
    return tk.asbool(
        tk.config.get(CONFIG_USE_FORWARDED_HOST, DEFAULT_USE_FORWARDED_HOST)
    )


def static_host() -> Optional[str]:
    return tk.config.get(CONFIG_STATIC_HOST, DEFAULT_STATIC_HOST)


def https() -> str:
    return tk.config.get(CONFIG_HTTPS, DEFAULT_HTTPS)


def use_nameid_as_email() -> bool:
    return tk.asbool(
        tk.config.get(CONFIG_USE_NAMEID_AS_EMAIL, DEFAULT_USE_NAMEID_AS_EMAIL)
    )



"""add attributes column to saml2_user table

Revision ID: e8e7ebedf90d
Revises: 25dc326c059e
Create Date: 2022-08-01 15:55:55.408354

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects.postgresql import JSONB

# revision identifiers, used by Alembic.
revision = "e8e7ebedf90d"
down_revision = "25dc326c059e"
branch_labels = None
depends_on = None


def upgrade():
    op.add_column(
        "saml2_user",
        sa.Column("attributes", JSONB, nullable=False, server_default="{}"),
    )
    pass


def downgrade():
    op.drop_column("saml2_user", "attributes")



import ckan.model as model
import ckan.plugins.toolkit as tk
import pytest

import ckanext.saml.helpers as h
from ckanext.saml import config
from ckanext.saml.model.user import User


@pytest.mark.usefixtures("with_plugins", "clean_db")
def test_is_saml_user(user):
    assert not tk.h.saml_is_saml_user(user["name"])

    model.Session.add(User(id=user["id"], name_id="test"))
    model.Session.commit()

    assert tk.h.saml_is_saml_user(user["name"])


@pytest.mark.usefixtures("with_plugins")
def test_login_button_text(ckan_config, monkeypatch, faker):
    assert tk.h.saml_login_button_text() == config.DEFAULT_LOGIN_TEXT

    label = faker.sentence()
    monkeypatch.setitem(ckan_config, config.CONFIG_LOGIN_TEXT, label)
    assert tk.h.saml_login_button_text() == label


@pytest.mark.usefixtures("with_plugins")
def test_folder_path(ckan_config, monkeypatch, faker):
    assert tk.h.saml_folder_path() == config.DEFAULT_FOLDER_PATH

    path = faker.file_path()
    monkeypatch.setitem(ckan_config, config.CONFIG_FOLDER_PATH, path)
    assert tk.h.saml_folder_path() == path


@pytest.mark.usefixtures("with_plugins")
def test_attr_mapper(ckan_config, monkeypatch, faker):
    assert tk.h.saml_attr_mapper() is None


@pytest.mark.usefixtures("with_plugins")
def test_settings(ckan_config, monkeypatch, faker):
    assert tk.h.saml_settings() == {}



import pytest

import ckan.model as model
from ckanext.saml.model import User


@pytest.mark.usefixtures("with_plugins", "clean_db")
class TestUser:
    def test_relationship(self, user):
        assert model.User.get(user["id"]).saml2_user is None

        su = User(id=user["id"], name_id="test")
        model.Session.add(su)
        model.Session.commit()

        assert su.user.name == user["name"]
        assert su.user.saml2_user == su

    def test_saml_user_use_cascade_remove(self, user):
        su = User(id=user["id"], name_id="test")
        model.Session.add(su)
        model.Session.commit()

        model.Session.delete(model.User.get(user["id"]))
        model.Session.commit()

        stored = model.Session.query(User).filter_by(id=su.id).one_or_none()
        assert stored is None

    def test_attributes(self, user):
        su = User(id=user["id"], name_id="test")
        model.Session.add(su)
        model.Session.commit()

        assert su.attributes == {}

        su.attributes = {"id": "test"}
        model.Session.commit()

        su = model.Session.query(User).filter_by(id=su.id).one()

        assert su.attributes == {"id": "test"}



import pytest
from ckan.tests import factories
from pytest_factoryboy import register


@pytest.fixture
def clean_db(reset_db, migrate_db_for):
    reset_db()
    migrate_db_for("saml")


@register
class UserFactory(factories.User):
    pass



from __future__ import annotations

import ckan.model as model
from sqlalchemy import Boolean, Column, UnicodeText
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, backref
from sqlalchemy.sql.schema import ForeignKey
from sqlalchemy.dialects.postgresql import JSONB

Base = declarative_base(metadata=model.meta.metadata)


class User(Base):
    __tablename__ = "saml2_user"

    id = Column(UnicodeText, ForeignKey(model.User.id), primary_key=True)
    name_id = Column(UnicodeText, nullable=False, unique=True)
    allow_update = Column(Boolean, default=False)
    attributes = Column(JSONB, nullable=False, default=dict)

    user = relationship(
        model.User,
        backref=backref(
            "saml2_user", uselist=False, cascade="all, delete-orphan"
        ),
    )



from __future__ import annotations

import json
import logging
import os
from typing import Any, Optional

import ckan.model as model
import ckan.plugins.toolkit as tk


from ckanext.saml.model.user import User
from ckanext.toolbelt.decorators import Collector
from . import utils, config

log = logging.getLogger(__name__)

helper, get_helpers = Collector("saml").split()


@helper
def logout_url(name_id: Optional[str] = None) -> str:
    req = utils.prepare_from_flask_request()
    auth = utils.make_auth(req)

    return auth.logout(
        return_to=tk.h.url_for("saml.post_logout", _external=True),
        name_id=name_id,
    )


@helper
def is_saml_user(name: str) -> bool:
    user = model.User.get(name)
    if not user:
        return False

    return model.Session.query(
        model.Session.query(User).filter_by(id=user.id).exists()
    ).scalar()


@helper
def login_button_text():
    return config.login_button_text()


@helper
def folder_path():
    return config.folder_path()


@helper
def attr_mapper():
    import importlib.util

    try:
        spec = importlib.util.spec_from_file_location(
            "module.name",
            tk.h.saml_folder_path()
            + "/attributemaps/"
            + tk.config.get("ckan.saml_custom_attr_map", "mapper.py"),
        )

        mapper = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(mapper)
    except Exception as e:
        log.error("{0}".format(e))
        return None

    return mapper.MAP


@helper
def settings() -> dict[str, Any]:
    custom_folder = tk.h.saml_folder_path()

    filepath = os.path.join(custom_folder, "settings.json")
    if not os.path.exists(filepath):
        log.warning("SAML2 settings file not found: %s", filepath)
        return {}

    with open(filepath) as src:
        settings_str = src.read()

    prefix = "ckanext.saml.settings.substitution."

    for k, v in tk.config.items():
        if not k.startswith(prefix):
            continue
        settings_str = settings_str.replace(f"<{k[len(prefix):]}>", v)
    settings = json.loads(settings_str)

    if config.use_remote_idp():
        settings["idp"] = tk.get_action("saml_idp_show")(
            {"ignore_auth": True}, {}
        )

    settings.setdefault("custom_base_path", custom_folder)
    return settings



"""add_foreign_key_constraint_on_user_id

Revision ID: 25dc326c059e
Revises: 92745f8a6168
Create Date: 2022-07-26 15:01:52.311445

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "25dc326c059e"
down_revision = "92745f8a6168"
branch_labels = None
depends_on = None


def upgrade():
    op.get_context().connection.execute(
        """
    DELETE FROM saml2_user
    WHERE id IN (
        SELECT s.id FROM saml2_user s
        LEFT JOIN "user" u ON u.id = s.id
        WHERE u.id IS NULL
    )"""
    )

    op.create_foreign_key(
        "saml2_user_id_fkey",
        "saml2_user",
        "user",
        ["id"],
        ["id"],
    )


def downgrade():
    op.drop_constraint(
        "saml2_user_id_fkey",
        "saml2_user",
        type_="foreignkey",
    )



from __future__ import annotations

import re
from typing import Any
from urllib.parse import urlparse

from onelogin.saml2.auth import OneLogin_Saml2_Auth, OneLogin_Saml2_Utils

import ckan.plugins as plugins
import ckan.plugins.toolkit as tk

from .interfaces import ICKANSAML
from . import config


def prepare_from_flask_request() -> dict[str, Any]:
    url_data = urlparse(tk.request.url)

    req_path = tk.request.path
    if tk.asbool(tk.config.get("ckan.saml_use_root_path", False)):
        # FIX FOR ROOT_PATH REMOVED IN request.path
        root_path = tk.config.get("ckan.root_path", None)
        if root_path:
            root_path = re.sub("/{{LANG}}", "", root_path)
            req_path = root_path + req_path

    host = tk.request.host
    static_host = config.static_host()
    forwarded_host = tk.request.environ.get("HTTP_X_FORWARDED_HOST")

    if config.use_forwarded_host() and forwarded_host:
        host = forwarded_host
    elif static_host:
        host = static_host

    return {
        "https": config.https(),
        "http_host": host,
        "server_port": url_data.port,
        "script_name": req_path,
        "get_data": tk.request.args.copy(),
        "post_data": tk.request.form.copy(),
    }


def make_auth(req: dict[str, Any]) -> OneLogin_Saml2_Auth:
    for p in plugins.PluginImplementations(ICKANSAML):
        Auth = p.saml_auth_class()
        if Auth:
            break
    else:
        Auth = OneLogin_Saml2_Auth

    if config.use_dynamic_config():
        return Auth(req, old_settings=tk.h.saml_settings())

    custom_folder = tk.h.saml_folder_path()
    return Auth(req, custom_base_path=custom_folder)


def decode_saml_response(response: str) -> bytes:
    return OneLogin_Saml2_Utils.decode_base64_and_inflate(response)
